package no.shhsoft.warchecker;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

/**
 * @author <a href="mailto:shh@thathost.com">Sverre H. Huseby</a>
 */
public final class WarData {

    private String name;
    private final Map<String, Set<Resource>> resources = new HashMap<String, Set<Resource>>();
    private final Map<String, Set<ResourceContainer>> containersByResourceName
        = new HashMap<String, Set<ResourceContainer>>();
    private final Set<ResourceContainer> containers = new HashSet<ResourceContainer>();
    private final ResourceContainer localContainer = new ResourceContainer("WEB-INF/classes", false);

    private void addResource(final Resource resource) {
        Set<Resource> set = resources.get(resource.getName());
        if (set == null) {
            set = new LinkedHashSet<Resource>();
            resources.put(resource.getName(), set);
        }
        set.add(resource);
    }

    private void addNameRef(final String name, final ResourceContainer container) {
        Set<ResourceContainer> refs = containersByResourceName.get(name);
        if (refs == null) {
            refs = new LinkedHashSet<ResourceContainer>();
            containersByResourceName.put(name, refs);
        }
        refs.add(container);
    }

    public WarData() {
    }

    public WarData(final String name) {
        setName(name);
    }

    public void setName(final String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public void addResource(final String name, final long size, final long crc, final long time,
                            final ResourceContainer container) {
        ResourceContainer theContainer = container;
        if (theContainer == null) {
            theContainer = localContainer;
        } else {
            containers.add(container);
        }
        final Resource resource = new Resource(name, size, crc, time);
        addResource(resource);
        theContainer.addResource(resource);
        addNameRef(resource.getName(), theContainer);
    }

    public Set<ResourceContainer> getContainers() {
        return containers;
    }

    public Set<Map.Entry<String, Set<ResourceContainer>>> getEntrySet() {
        return containersByResourceName.entrySet();
    }

    public Set<Resource> findResourcesByName(final String name) {
        return resources.get(name);
    }

    public Set<ResourceContainer> findContainersByResourceName(final String name) {
        return containersByResourceName.get(name);
    }

    public String[] getResourceNames() {
        final Set<String> names = resources.keySet();
        final String[] ret = new String[names.size()];
        int idx = 0;
        for (final String name : names) {
            ret[idx++] = name;
        }
        Arrays.sort(ret);
        return ret;
    }

    public String[] getContainerNames() {
        final String[] ret = new String[containers.size()];
        int idx = 0;
        for (final ResourceContainer container : containers) {
            ret[idx++] = container.getName();
        }
        Arrays.sort(ret);
        return ret;
    }

}
