package no.shhsoft.warchecker.maven;

import java.io.File;
import java.util.List;

import no.shhsoft.warchecker.WarChecker;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

/**
 * @goal check
 * @author <a href="mailto:shh@thathost.com">Sverre H. Huseby</a>
 */
public final class MavenWarCheckerPlugin
extends AbstractMojo {

/* usage example */
//<build>
//    <plugins>
//        <plugin>
//            <groupId>no.shhsoft</groupId>
//            <artifactId>WarChecker</artifactId>
//            <version>1.0-SNAPSHOT</version>
//            <executions>
//                <execution>
//                    <phase>package</phase>
//                    <goals>
//                        <goal>check</goal>
//                    </goals>
//                </execution>
//            </executions>
//        </plugin>
//    </plugins>
//</build>


    /**
     * @parameter expression="${project.build.directory}/${project.build.finalName}.war"
     */
    private String warFile;

    /**
     * @parameter
     */
    private List<String> additionalJars;

    /**
     * @parameter expression="true"
     */
    private boolean ignoreMissingJars;

    /* TODO: allow directing output to a file. see MavenOut.setInfoPrintWriter */

    public void execute()
    throws MojoExecutionException {
        final File file = new File(warFile);
        if (!file.isFile()) {
            throw new MojoExecutionException(
                "no file `" + file + "'. please specify a `warFile' property giving the "
                + "name of the war file in the `configuration' section of this plugin.");
        }
        final WarChecker warChecker = new WarChecker(new MavenOut(getLog()));
        warChecker.checkWarFile(warFile, additionalJars, ignoreMissingJars);
    }

}
