package no.shhsoft.warchecker.reporters;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Set;

import no.shhsoft.warchecker.Indenter;
import no.shhsoft.warchecker.Reporter;
import no.shhsoft.warchecker.Resource;
import no.shhsoft.warchecker.ResourceContainer;
import no.shhsoft.warchecker.WarData;

/**
 * @author <a href="mailto:shh@thathost.com">Sverre H. Huseby</a>
 */
public abstract class AbstractReporter
implements Reporter {

    private static final SimpleDateFormat ISO_DATE = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
    private static boolean skipResourceComparison = false;
    private static boolean includeTimeInResourceComparison = false;

    protected AbstractReporter() {
    }

    protected static final String adjust(final String s, final int len, final boolean fillLeft) {
        final String spaces = Indenter.getSpaces(len - s.length());
        if (fillLeft) {
            return spaces + s;
        }
        return s + spaces;
    }

    protected static final String getContainerNameAndResourceDataText(
                                                                      final ResourceContainer container,
                                                                      final String resourceName) {
        final Resource resource = container.findResourceByName(resourceName);
        final StringBuilder sb = new StringBuilder();
        sb.append(adjust(container.getName(), 30, false));
        sb.append(" (size:");
        sb.append(resource.getSize() >= 0L ? adjust(Long.toString(resource.getSize()), 5, true)
                                          : "?");
        sb.append(", crc: ");
        sb.append(resource.getCrc() != -1L ? adjust(Long.toHexString(resource.getCrc()), 8, true)
                                          : "?");
        sb.append(", ");
        sb.append(ISO_DATE.format(new Date(resource.getTime())));
        sb.append(")");
        return sb.toString();
    }

    protected static final boolean consideredEqual(final Resource r1, final Resource r2) {
        if (!r1.getName().equals(r2.getName())) {
            return false;
        }
        if (r1.getSize() != r2.getSize()) {
            return false;
        }
        if (r1.getCrc() != r2.getCrc()) {
            return false;
        }
        if (includeTimeInResourceComparison && r1.getTime() != r2.getTime()) {
            return false;
        }
        return true;
    }

    protected static final boolean allEqual(final Set<ResourceContainer> refs,
                                            final String resourceName) {
        if (skipResourceComparison) {
            return false;
        }
        Resource first = null;
        for (final ResourceContainer container : refs) {
            final Resource resource = container.findResourceByName(resourceName);
            if (first == null) {
                first = resource;
            } else {
                if (!consideredEqual(first, resource)) {
                    return false;
                }
            }
        }
        return true;
    }

    public static void setSkipResourceComparison(final boolean skipResourceComparison) {
        AbstractReporter.skipResourceComparison = skipResourceComparison;
    }

    public static boolean isSkipResourceComparison() {
        return skipResourceComparison;
    }

    public static void setIncludeTimeInResourceComparison(final boolean includeTimeInResourceComparison) {
        AbstractReporter.includeTimeInResourceComparison = includeTimeInResourceComparison;
    }

    public static boolean isIncludeTimeInResourceComparison() {
        return includeTimeInResourceComparison;
    }

    public abstract String report(final WarData warData);

}
