package no.shhsoft.warchecker.reporters;

import java.util.Set;

import no.shhsoft.warchecker.Indenter;
import no.shhsoft.warchecker.Resource;
import no.shhsoft.warchecker.ResourceContainer;
import no.shhsoft.warchecker.WarData;

/**
 * @author <a href="mailto:shh@thathost.com">Sverre H. Huseby</a>
 */
public final class ContainerResourceOverrideReporter
extends AbstractReporter {

    private static final String[] CONTAINER_RESOURCES = {
        "java/lang/Object.class",
        "javax/ejb/EJBHome.class",
        "javax/servlet/Servlet.class",
        "javax/servlet/jsp/JspPage.class",
    };

    @Override
    public String report(final WarData warData) {
        final StringBuilder sb = new StringBuilder();
        sb.append("Classpath resources that override selected container resources\n");
        sb.append("--------------------------------------------------------------\n");
        sb.append('\n');
        for (final String resourceName : CONTAINER_RESOURCES) {
            final Set<Resource> resources = warData.findResourcesByName(resourceName);
            if (resources != null) {
                boolean problemFound = false;
                for (final ResourceContainer container
                         : warData.findContainersByResourceName(resourceName)) {
                    if (container.isExternal()) {
                        continue;
                    }
                    if (!problemFound) {
                        sb.append(Indenter.getIndentSpaces());
                        sb.append(resourceName);
                        sb.append('\n');
                        problemFound = true;
                    }
                    sb.append(Indenter.getIndentSpaces(2));
                    sb.append(getContainerNameAndResourceDataText(container, resourceName));
                    sb.append('\n');
                }
                if (problemFound) {
                    sb.append('\n');
                }
            }
        }
        return sb.toString();
    }

}
