package no.shhsoft.warchecker.reporters;

import java.util.Set;

import no.shhsoft.warchecker.Indenter;
import no.shhsoft.warchecker.ResourceContainer;
import no.shhsoft.warchecker.WarData;

/**
 * @author <a href="mailto:shh@thathost.com">Sverre H. Huseby</a>
 */
public final class ResourceDuplicateReporter
extends AbstractReporter {

    private static final String[] SKIPPABLE_PATTERNS = {
        "META-INF/MANIFEST.MF",
        "META-INF/NOTICE.txt",
        "LICENSE",
        "LICENSE.txt",
        "NOTICE",
        "NOTICE.txt",
        "META-INF/LICENSE",
        "META-INF/LICENSE.txt",
        "META-INF/INDEX.LIST",
        "overview.html",
    };

    private static boolean isSkippable(final String name) {
        for (final String re : SKIPPABLE_PATTERNS) {
            if (name.matches(re)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public String report(final WarData warData) {
        final StringBuilder sb = new StringBuilder();
        sb.append("Classpath resources that appear multiple times\n");
        sb.append("----------------------------------------------\n");
        sb.append('\n');
        for (final String resourceName : warData.getResourceNames()) {
            if (isSkippable(resourceName)) {
                continue;
            }
            final Set<ResourceContainer> refs = warData.findContainersByResourceName(resourceName);
            if (refs.size() > 1) {
                if (allEqual(refs, resourceName)) {
                    continue;
                }
                sb.append(Indenter.getIndentSpaces());
                sb.append(resourceName);
                sb.append('\n');
                for (final ResourceContainer container : refs) {
                    sb.append(Indenter.getIndentSpaces(2));
                    sb.append(getContainerNameAndResourceDataText(container, resourceName));
                    sb.append('\n');
                }
                sb.append('\n');
            }
        }
        return sb.toString();
    }

}
