package no.shhsoft.warchecker.reporters;

import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

import no.shhsoft.warchecker.Indenter;
import no.shhsoft.warchecker.WarData;

/**
 * @author <a href="mailto:shh@thathost.com">Sverre H. Huseby</a>
 */
public final class SimilarlyNamedJarsReporter
extends AbstractReporter {

    private static String simplify(final String s) {
        return s.toLowerCase()
            .replaceAll("-snapshot", "")
            .replaceAll("[_-]nodep", "")
            .replaceAll("[_-]beta", "")
            .replaceAll("([0-9_-])rc([0-9.-])", "$1$2")
            .replaceAll("\\d+(\\.\\d+)+", "")
            .replaceAll("([_-])\\d\\d+", "$1")
            .replaceAll("_\\d+", "")
            .replaceAll("\\.[a-zA-Z]([^a-zA-Z])", "$1")
            .replaceAll("--+", "-")
            .replaceAll("__+", "_")
            .replaceAll("_-", "_")
            .replaceAll("-_", "-")
            .replaceAll("[_-]\\.", ".");
    }

    private Map<String, Set<String>> getSimplifiedNames(final WarData warData) {
        final Map<String, Set<String>> map = new LinkedHashMap<String, Set<String>>();
        for (final String name : warData.getContainerNames()) {
            final String simplified = simplify(name);
            Set<String> set = map.get(simplified);
            if (set == null) {
                set = new HashSet<String>();
                map.put(simplified, set);
            }
            set.add(name);
        }
        return map;
    }

    @Override
    public String report(final WarData warData) {
        final StringBuilder sb = new StringBuilder();
        sb.append("Jar files with similar names\n");
        sb.append("----------------------------\n");
        sb.append('\n');
        final Map<String, Set<String>> map = getSimplifiedNames(warData);
        for (final Map.Entry<String, Set<String>> entry : map.entrySet()) {
            final Set<String> set = entry.getValue();
            if (set.size() > 1) {
                for (final String name : set) {
                    sb.append(Indenter.getIndentSpaces());
                    sb.append(name);
                    sb.append('\n');
                }
                sb.append('\n');
            }
        }
        return sb.toString();
    }

}
