package no.shhsoft.warchecker.reporters;

import no.shhsoft.warchecker.Indenter;
import no.shhsoft.warchecker.WarData;

/**
 * @author <a href="mailto:shh@thathost.com">Sverre H. Huseby</a>
 */
public final class SuspiciousJarsReporter
extends AbstractReporter {

    private static final String[] SUSPICIOUS_JAR_PATTERNS = {
        ".*-mock.*",
        "junit.*",
        ".*-test.*",
        ".*debug.*",
        ".*jetty.*",
        ".*tomcat.*",
        ".*weblogic.*",
        ".*websphere.*",
    };

    private static boolean matchesSuspiciousJarPattern(final String name) {
        for (final String re : SUSPICIOUS_JAR_PATTERNS) {
            if (name.matches(re)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public String report(final WarData warData) {
        final StringBuilder sb = new StringBuilder();
        sb.append("Jar files with suspicious names\n");
        sb.append("-------------------------------\n");
        sb.append('\n');
        for (final String name : warData.getContainerNames()) {
            if (!matchesSuspiciousJarPattern(name)) {
                continue;
            }
            sb.append(Indenter.getIndentSpaces());
            sb.append(name);
            sb.append('\n');
        }
        return sb.toString();
    }

}
